package Addressbook;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Group.java,v 1.1.1.1 2001/12/06 19:33:52 bradford Exp $
 */

import java.util.Vector;
import java.util.Hashtable;

import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;

import org.w3c.dom.*;
import org.apache.xpath.*;
import org.apache.xpath.objects.*;

/**
 * Class for holding groups of the Person class
 */
public class Group {

   private Vector personList = new Vector();
   private int index = 0;

   /**
    * Returns current index
    */
   public int getIndex() {
      return index - 1;
   }

   /**
    * Return the next person from the list
    */
   public Person getNext() {
      Person person = (Person) personList.get(index);
      index = index + 1;
      return person;
   }

   /**
    * Return the person element at a certain index
    */
   public Person getPersonAt(int i) {
      return (Person) personList.get(i);
   }

   /**
    * Add person to Group
    */
   public void add(Person person) {
      personList.add(person);
   }

   /**
    * Add Persons from Result set
    */
   public void addResults(ResourceIterator results) {
      try {
         while (results.hasMoreResources()) {

            // Create a new person instance to add to Group
            Person person = new Person();

            // Retrieve the next node
            XMLResource resource = (XMLResource) results.nextResource();
            // Cast to Dom Node
            Node originalnode = resource.getContentAsDOM();

            // Get the document key for this node for later use
            String dockey = resource.getDocumentId();
            person.setDocKey(dockey);

            // Use Xalan xpath parser to extract data, store into hastable
            XObject xo = new XObject();

            xo = XPathAPI.eval(originalnode, "/person/fname");
            person.setFname(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/lname");
            person.setLname(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/phone[@type='work']");
            person.setWorkPhone(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/phone[@type='home']");
            person.setHomePhone(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/phone[@type='cell']");
            person.setCellPhone(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/email[@type='home']");
            person.setHomeEmail(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/email[@type='work']");
            person.setWorkEmail(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/address[@type='home']");
            person.setHomeAddress(xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/address[@type='work']");
            person.setWorkAddress(xo.toString());

            // Add person to group
            this.add(person);
         }
      }
      catch (Exception e) {
         e.printStackTrace();
      }

   }

   /**
    * Remove Person from Group
    */
   public void remove(int i) {
      personList.remove(i);
   }

   /**
    * Remove all Persons from Group
    */
   public void removeAll() {
      personList.removeAllElements();
   }

   /**
    * Return the size of the Group
    */
   public int sizeOf() {
      return personList.size();
   }

   /**
    * Reset counter for Person retrieval
    */
   public void reset() {
      index = 0;
   }

   /**
    * Return true if more persons exist
    */
   public boolean hasMorePersons() {
      return index < personList.size();
   }
}

