package org.apache.xindice.server;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Versioning.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import org.apache.xindice.util.*;

/**
 * Versioning is the standard identification, versioning, and
 * namespacing interface for Xindice resources. In order to avoid resource
 * naming collisions between multiple applications, Xindice dynamically
 * creates packages (namespaces) matching the value returned by a resource's
 * getPackageName method. Resources stored within these namespaces can
 * only be seen by modules who return the same package name when queried by the
 * ResourceManager. Xindice acknowledges the existence of a common,
 * shared package called the null package. Resource related methods are usually
 * provided with two prototypes, one that accepts a Versioning
 * reference, and one that doesn't. The method that doesn't accept a
 * Versioning reference resolves to this null package. Resources stored
 * in the null package can be manipulated by any module. The String
 * value for the null package is an empty String ("").
 */

public interface Versioning extends Named {
   /**
    * getVersion returns the version number for this package.  It is
    * conceivable that a single instance of Xindice may be running two
    * different versions of the same appliction.  getVersion allows
    * an added ambiguity check to be performed in these cases.
    *
    * @return The package version
    */
   float getVersion();

   /**
    * getPackageName returns the package name for this Versioning
    * object.
    *
    * @return The package name
    */
   String getPackageName();
}
