package org.apache.xindice.tools.command;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: DeleteIndexer.java,v 1.1.1.1 2001/12/06 19:33:57 bradford Exp $
 */

import java.util.Hashtable;
import java.io.*;

import org.xmldb.api.base.*;
import org.xmldb.api.DatabaseManager;

import org.apache.xindice.client.xmldb.*;
import org.apache.xindice.client.xmldb.services.*;


/**
 * DeleteIndexer.java is designed to let the user delete or drop
 * an Indexer from a named Collection or a nested Collection within
 * the Database.
 */

public class DeleteIndexer extends Command {
    public boolean execute(Hashtable table) throws Exception {

        Collection col = null ;

        try {

                BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

                if (table.get("collection") != null) {
                    //CollectionManager manager = db.getCollection((String)table.get("collection")).getCollectionManager();
                    // Get a Collection reference to the collection
                    String colstring = normalizeCollectionURI( (String)table.get("collection") );
                    col = DatabaseManager.getCollection( colstring );
                    if ( col == null ) {
                        System.out.println("ERROR : Collection not found!");
                        return false;
                    }

                    // Create a collection manager instance for the collection
                    CollectionManager colman = (CollectionManager)col.getService("CollectionManager",XMLDBAPIVERSION);

                    if (table.get("nameOf") != null) {
                        if (!table.get("force").equals("yes")) {
                            // Allow the user to abort the deletion process
                            System.out.println("Continue deletion process? (y/n)");
                            try {
                                final String ans = in.readLine();

                                if (!ans.equalsIgnoreCase("y")) {
                                    System.out.println("Operation aborted.");
                                    return true;
                                }

                            } catch (IOException ie) {
                                System.out.println(ie);
                                return false;
                            }
                        }

                        colman.dropIndexer((String)table.get("nameOf"));
                        System.out.println("DELETED: " + (String)table.get("nameOf"));
                    }
                    else {
                        System.out.println("ERROR : Name switch and Indexer name required");
                    }
                }
                else
                    System.out.println("ERROR : Collection and switch required");

        } catch (Exception e) {
            System.out.println("ERROR : " + e.getMessage() );
            if (table.get("verbose").equals("true")) {
              org.apache.xindice.Debug.printStackTrace(e);
            }
            return false;
            
        } finally {
            if ( col != null ) {
                col.close();
            }
        }

      return true;
   }

}

